package de.ksqaured.jds.components.plugin;

import static de.ksquared.jds.Utilities.isNull;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.beans.PropertyVetoException;
import java.util.Map;

import de.ksquared.jds.Utilities;
import de.ksquared.jds.components.Component;
import de.ksquared.jds.components.Component.HiddenComponent;
import de.ksquared.jds.components.Configurable;
import de.ksquared.jds.components.Interactable;
import de.ksquared.jds.components.Sociable;
import de.ksquared.jds.contacts.Contact;
import de.ksquared.jds.contacts.ContactList;
import de.ksquared.jds.contacts.ContactList.Alignment;
import de.ksquared.jds.contacts.InputContact;
import de.ksquared.jds.contacts.OutputContact;

/** Example-Plugin Component Full
 * This example was developed to show and describe every aspect of the JDigitalSimulator
 * plugin development shortly, but fully covered. Due to the complete object oriented approach
 * of the JDigitalSimulator is is very easy to create and deploy new plugin-Components to the
 * JDigitalSimulator.
 * It was tried to explain a new behaviour and/or feature of the JDigitalSimulator in every single
 * line of coding. Due to this many coding fragments are may not requried to create the plugin
 * of your choice. The minimum plugin requirements are very simple: extend Component. Everything
 * else is optional, but provides many additional features.
 * I will mark every important/required step in this tutorial with an //XXX marker.
 * Have fun coding an own plugin, so I had while devloping the JDigitalSimulator.
 * Regards,
 * @author Kristian Kraljic (kriskra@gmail.com)
 */

/** Please make sure you have imported the <code>JDigitalSimulator.jar</code> to your classpath //XXX
 * so you will get nice value-helps and some documentations for all of the methods available.
 */

/** The annotation HiddenComponent means, that this component is not selectable in the catalog **/
@HiddenComponent
/** There is basically one requirement to develop your own component:
 *   <code>extend de.ksquared.jds.components.Component</code>. //XXX
 *   So you will have to implement the following methods:
 *     - <code>paint</code>: Paints the component to a graphics-Object
 *     - <code>getSize</code>: Has to return the actual component dimension
 *     - <code>calculate</code>: Calculates the required information for this component
 *   Simple, isn't it? But we will go into detail for each of those methods.
 * Following interfaces are optional, but provide (mostly needed) features:
 *   - <code>Sociable</code>: Means that this component can be connected to other components (via wires)
 *   - <code>Interactable</code>: Means that this component has events to interact with the user (clicks)
 *   - <code>Configurable</code>: Means that this component can be configured, so it has an configuration dialog
 * But, guess what... We will go into detail of those aswell.
 */
public class ExamplePluginFull extends Component implements Sociable,Interactable,Configurable {
	/** The first very small, but indeed very important, information is, that every component
	 * must be <code>Serializable</code>, necessary when the component is copied, cloned or stored. //XXX
	 * This means, every single non-static, non-transient field will get stored into a file
	 * or the memory, even if you do not intend to have your component physically stored.
	 * So just use serializable classes or make sure the required field are transient. You may also
	 * make use of the writeObject/readObject methods as described in the Serializable Java Docs.
	 * You should put serialVersionUID into each of your plugins aswell.
	 */
	private static final long serialVersionUID = 1l;

	/** The second very small, but again very important, information you will have to provide is
	 * the obligatory field <code>public static final ComponentAttributes componentAttributes</code> //XXX
	 * This field contains information about the Component such as the name and group entry.
	 * Following informations may be provided to enable JDigitalSimulator to get the required info:
	 * 	 <code>new ComponentAttributes(</code>
	 *     - name: The name of the component. This name will get displayed in the JDigitalSimulator
	 *             and is also used as unique identifier. So make sure there is no similar plugin
	 *             already existing
	 *     - group: The group of the component to categorize it in the menu. The following default
	 *              groups are available: group.general, group.display, group.switch, group.gate,
   *                                    group.flipflop, group.register, group.selector, group.alu,
   *                                    group.counter, group.converter, group.interface, 
   *                                    group.additional, group.others or any other group name.
	 *     - description: (optional) The description should provide more detailed information.
	 *     - author: (optional) The author of this plugin Component (may with some additional info-
	 *               rmation, like your homepage or contact information.
	 *     - version: (optional) The actual version of this component to track the changes.
	 *   <code>);</code>
	 * The constructors of <code>ComponentAttributes</code> will guide you through the information
	 * which is necersarry to create an appropriate <code>componentAttributes</code> attribute.
	 */
	public static final ComponentAttributes componentAttributes = new ComponentAttributes("Example-Plugin","group.general","This is an template component for how to create an own plugin.","Kristian Kraljic (kriskra@gmail.com)",1);
	
	/** Let us implement our first method. So we will have to
	 *    <code>@Override public void paint(Graphics graphics)</code> //XXX
	 * of the <code>Paintable</code> interface.
	 * This method is used to draw the whole component and is called for the first time when
	 * generating a preview image for the component-catalog of the JDigitalSimulator.
	 * You may use every possibility to paint your component provided by the java
	 * <code>Graphics2D</code> classes, as described in the java documentation.
	 * It is strictly recommended that you only draw into the bounds of your own Component! It is
	 * a known-feature that you are able to draw outside of your component bounds (returned by
	 * <code>getSize</code>, but do not do it if you do not know exactly what you are doing.
	 * Note: Do <bold>not</bold> use this method for calculating anything! This method will be
	 *       likely get called when something changes on the frontend, but it is not guranteed in
	 *       any ways, that it: a) is called every time b) called in fixed time intervalls.
	 * Please use the calculate method for this reason, described later on.  
	 */
	@Override public void paint(Graphics graphics) {
		Dimension size = getSize(); /** you may call other (even private) methods in here! */
		graphics.setColor(Color.BLACK); /** do not assume the color does fit right away */
		graphics.drawRect(0,0,size.width,size.height); /** only draw inside this bounds! */
		graphics.drawString(getAttributes().name,0,30);
		/** there is no need to call the super.paint method for any reason */
	}
	
	/** The second method we will have to overwrite is the
	 *   <code>@Override public Dimension getSize()</code> //XXX
	 * method of the <code>Sizable</code> interface.
	 * This method is used to determine the bounds of your application and is later used whenever
	 * your component gets placed, dragged, moved or displayed. So assume this method gets called
	 * quite often while it is getting displayed in the JDigitalSimulator.
	 * Note: Please do <bold>never</bold> return null, or stupid values like: 0, -1, Integer.MAX_VALUE
	 * (You will not have to care about any zoom or move-action. This is handled by the framework)
	 */
	@Override public Dimension getSize() {
		Dimension dimension = new Dimension(80,80); /** create a new suitable size for your component */
		boolean niceWeather = System.currentTimeMillis()%10000>9000; /** you may do basic calculations */
		if(niceWeather) dimension.height += 20; /** and change the size of your component dynamically
		                                            (even at runtime! or based on user configuration. */
		return dimension; /** but you will have to return a correct dimension anyways! */
	}
	
	/** Now we will have to discuss the benefits of the <code>Sociable</code> interface. Most times
	 * we want to create a component which may gets connected to other components using wires.
	 * (some components may not be connected to others like text or images). For this reason we
	 * will have to implement the <code>Sociable</code> interface. This will lead to that we have to
	 *   <code>@Override public Contact[] getContacts()</code> //XXX
	 * which has to return all <code>Contact</code>s of this Component. Wait, wait, wait? What the
	 * heck is a contact? A contact is the Object which is used to communicate to other components.
	 * You will not have to care about any wire, you will just have to create a interface for wire's
	 * using Contacts. Therefore there are two standard implementations of the contact-class:
	 *   - <code>InputContact</code>: Used to receive values from other components
	 *                                (Note: only one wire is connectable to an InboundContact)
	 *   - <code>OutputContact</code>: Used to set the results of your calculations and make them
	 *                                 available to other components.
	 *                                 (Note: Several wires may be connected to one OutputContact)
	 * You component may use both of them as desired in any combination. Each Contact has got a
	 * unique location in your component, so it is recommened to initialize it right when the
	 * component gets created in the constructor. But for this example we will do it in the
	 * <code>getContacts</code> method directly, to show that also a dynamic ammount of Contacts is
	 * possible. To explain every detail of what a Contact is capable of would take way to long. So
	 * you have to know only a few methods to work with Contacts right away.
	 *   - <code>isCharged</code>: For Input- and OutputContacts //XXX
	 *   - <code>setCharged</code>: Set the Output-Contact as charged or uncharged
	 *   - <code>setLocation</code>: To set the location of your contact (relative in your Component)
	 * Note: You may use the class <code>ContactList</code> to create a dynamic amount of contacts,
	 *       and place them in your Component accordingly. The class also provides some additional
	 *       usefull Methods to work with Conacts like the static methods <code>concatenateContacts</code>
	 *       or <code>setContactLocations</code>.
	 * So lets begin by creating some new instance variables to store the Contacts we will create.
	 */
	private InputContact input; /** create a single input contact */
	private OutputContact[] outputs; /** some more output contacts as an array of contacts */
	private ContactList<OutputContact> moreOutputs; /** and using the contact list, even more outputs */	
	/** Note: only the contacts which will be returned by this method are available in your component
	 *        later on. Please to not experiment with this method, simly return all of the contacts
	 *        of your component, no matter what or where there are placed. */
	@Override public Contact[] getContacts() {
		if(Utilities.isNull(input)) { /** use the utility methods of the class <code>Utilities</code> */
			/** create a new input contact, you will have to pass your component and a default loction */
			input = new InputContact(this,new Point(10,10));
			input.setLocation(new Point(20,20)); /** you may move the contact arround later on */
			input.isCharged(); /** a input contact is charged by default if no wire is connected */
			input.isWired(); /** you may check if there are already wires connected using this method */
		}
		if(outputs==null||outputs.length==0) { /** please be carefull with array to not generate any exceptiosn */
			/** create a new array of output contacts, we will arange them later on automatically */
			outputs = new OutputContact[] { new OutputContact(this), new OutputContact(this) };
			/** pleace all the created output contacts dynamically in an horizontal aligment at the
			 * top of the component. You may use other constructors to modify the pacement of the Contacts */	
			ContactList.setContactLocations(this,outputs,Alignment.HORIZONTAL);
		}
		if(isNull(moreOutputs)) { /** I would even recommend to use static imports for some methods */
			/** if you are creating a ContactList, you will have to pass a class to generate the Contacts */
			moreOutputs = new ContactList<OutputContact>(this,OutputContact.class);
			moreOutputs.setContacts(3); /** you can simply create a dynamic number of contacts by using
			                                the <code>setContacts</code> method of the ContactList class */
			moreOutputs.setContactLocations(); /** don't forget to place the contacts in your component */
			moreOutputs.getContact(0).setCharged(); /** you may charge some of your outputs by default */
		}
		/** finally we have to return an array of all created contacts to the JDigitalSimulator, so
		 * the framework knows how many of them are available, and how to interact with this plugin.
		 * Note: Check out the <code>concatenateContacts</code> methods of the ContactList class.
		 */
		return ContactList.concatenateContacts(ContactList.concatenateContacts(input,outputs),
				                                   moreOutputs.toArray()); /** convert it to an array first */
	}
	
	/** the last method I want to describe in detail is the calculate method. It should be used
	 * for all calculations of your component. So at the best (and recommended case) every interaction
	 * of Contacts, so setting and getting Contact charges, should be done <bold>only</bold> in here.
	 * To enable a great possibility to caculate your component it is absolutely guranteed that
	 *   the <code>calculate</code> method is called once every 10th of a second by the Simulation //XXX
	 * once the simulation has started. 
	 */
	@Override	public void calculate() {
		outputs[0].setCharged(input.isCharged());
		outputs[1].setCharged(!input.isCharged());
		OutputContact contact = moreOutputs.getContact(0);
		contact.setCharged(!contact.isCharged());
		if(moreOutputs.getContact(1).isWired())
			moreOutputs.getContact(2).setCharged();
	}
	
	//TODO
	@Override	public Option[] getOptions() { return null; }
	@Override public Map<Option, Object> getConfiguration() { return null; }
	@Override public void setConfiguration(Map<Option, Object> configuration) throws PropertyVetoException {}

	@Override	public void mouseClick(MouseEvent event) {}
	@Override public void mouseDoubleClick(MouseEvent event) {}
	@Override public void mouseDown(MouseEvent event) {}
	@Override	public void mouseUp(MouseEvent event) {}
}

/** Finally we are through our small tutorial. Now we need to test our component in the
 * JDigitalSimulator. Even this is way to easy to be true. ;-) Simply place the class file of
 * our component or any jar file containing your component into the "plugins"-directory of
 * your JDigitalSimulator (were you have propably found this file aswell) and start the
 * JDigitalSimulator. If you have set the componentAttributes field correctly the component
 * will get directly displayed in the menu of the JDigitalSimulator. */

/* I hope you enjoyed my small tutorial, if you have any further questions or comments please
 * feel free to contact me at kriskra@gmail.com. Thanks and best regards, Kristian Kraljic */
//   COPYRIGHT 2010 (c) - Kristian Kraljic - kSquared Development (k)   //